Documentation
¶
Overview ¶
Package watch provides a file watcher that serves DOT graphs as SVG via HTTP.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type Config ¶
type Config struct {
File string // DOT file to serve
Port string // HTTP server port (use "0" for a random available port)
Debug bool // enable debug logging
Stdout io.Writer // output for status messages
Stderr io.Writer // output for error logging
}
Config configures a Watcher.
type Watcher ¶
type Watcher struct {
// contains filtered or unexported fields
}
Watcher watches a DOT file for changes and serves the rendered SVG via HTTP. It provides an SSE endpoint that notifies connected browsers when the file changes.
Click to show internal directories.
Click to hide internal directories.